; DGPS Radio Controller Software -- PIC16C73B, 16C76
; Copyright (c) 1998, 1999 Rich Heineck, All rights reserved.
; This program can be freely used for non-commercial purposes, and carries no warranty of any kind.
;
; File:		Demod.asm
; Date:		04/30/98
; Language:	Microchip 16CXX Assembler
;
;   This module contains the routines for demodulating the raw data signal. It contains the 
;   CCP (input capture) and Timer2 ISR's and the code for generating a serial bit stream 
;   representing the RTCM-104 data.
;
;  Change History
;	6/9/98	  Added station LOCK detection logic to timer2_isr.
;	6/14/99   Added support for 16C73B - eliminated shared data space.
;		  Changed period calculation to clip at the 8-bit boundaries
;		  rather then wrap around if the signal exceeds the 8-bit range. This
;		  should improve the bit error rate in the presence of noise.
;------------------------------------------------------------------------------
	INCLUDE		<pic_hdr.inc>
	INCLUDE		<dgps.inc>
	LIST		n=114, x=ON


	EXTERN	ISR_return, CurrentBPS, PslibFlags, CurrentFreq, CurrentBPS
	GLOBAL	Cap1_isr, Cap2_isr, timer2_isr, InitDemod, SetBitRate


	CONSTANT	CAPTURE_FE = B'0100'
	CONSTANT	CAPTURE_RE = B'0101'



;-- Timer1 is set up to increment every 2 us

INI_T1CON	EQU	1<<T1CKPS0 | 1<<TMR1ON		; Internal clock, 1:2 prescaler (Fosc = 4 MHz)
INI_CCP1CON	EQU	CAPTURE_FE
INI_CCP2CON	EQU	CAPTURE_RE

US_PER_TIC	EQU	2	; 2 us per timer tic
BASE_PER_CNT	EQU	D'800'	; All period count values normalized to 800us offset
MIN_DT		EQU	BASE_PER_CNT / US_PER_TIC	; lower "clip" limit
MAX_DT		EQU	MIN_DT + D'255'			; upper clip limit



;-- Values defining the symbol (bit) clock. The reference time base is timer2, which generates 
;   interrupts every 2.496 ms. The postscaler must remain at 1:1. See design description for
;   required changes if Fosc is not 4 MHz.

INI_T2CON	EQU	1<<TMR2ON | 1<<T2CKPS1		; 16:1 prescale, 1:1 postscale
TMR2_CNT	EQU	D'156'				; PR2 = value - 1

RTCM_BITS	EQU	6		; Number of data bits per RS-232 byte (RTCM-104 std)


;-- Module Variables --
;
;  'N_bitrate' is a multiplier for the bit clock reference (Timer2 interrupts) that sets the 
;	data bit rate. It is initialized at station tuning time and remains fixed. The current
;	design requires this to be an even integer so the bit clock can be evenly split in two.
;  'Phase' is the bit clock time stamp made by the bit transition detection logic, and is used 
;	in calculating the bit clock phase error. Resolution is same as Timer2.
;  'DataSum' is used to determine the most likely state of the current data bit, which is done 
;	by summing the differences of each measured signal sample period and the baseline 
;	carrier period.

BANK0	UDATA

N_bitrate	RES	1	; Bit clock reference multiplier that sets bit period
BitClock	RES	2	; 16-bit symbol (bit) clock timer. Resolution is same as Timer2.
Phase		RES	2	; (see comment block, above)
HalfTime	RES	2	; Set to 1/2 the bit period (Timer2 resolution).
DataSum		RES	2	; Signed accumulator for bit state acquisition

tmr2ext		RES	1	; Timer2 extension. Counts 2.5 ms interrupts during each bit period.
TxChar		RES	1	; Byte that gets built up with incoming data
BitCnt		RES	1	; Tracks number of bits stored in TxChar
LockDet		RES	1	; Used in determining if station is locked
MaxLockErr	RES	1	; Initialized to value used in determining if PLL phase error exceeds "station lock" range.

DataFlags	RES	1	; Flag register for raw data & bit status
DATA_STATE	EQU	0	; Bit in DataFlags: Stores current logic state of data stream
TRANSITION	EQU	1	; Bit in DataFlags: Flag indicating bit level changed and phase adjustment should be made.
BIT_LEVEL	EQU	3	; Bit in DataFlags: Stores most recent data bit state

LastICL		RES	1	; Stores previous captured timer LSB. Used to calculate period.
LastICH		RES	1	; Previous timer MSB
AvePeriod	RES	1	; Averaged period from last 4 measurements
AveSum		RES	2	; 16 bit sum for signal averaging
PerStk		RES	4	; Stack used to store last 4 raw period measurements

INIT_BASELINE	EQU	((D'1000' - BASE_PER_CNT) / US_PER_TIC)	; period count for 500 Hz half cycle (1000 us)
INIT_BASE_SUM	EQU	INIT_BASELINE * D'1024'	; Number of samples for average calculation is fixed at 1024.

BaseSum		RES	3	; 24 bit sum used in Baseline calculation
Baseline	RES	1	; Calculated Baseline period for unmodulated carrier (500 Hz nominal)

tmp0		RES	1	; temporary storage - local variables
tmp1		RES	1
tmp2		RES	1
tmp3		RES	1

tmp4		RES	1
tmp5		RES	1

	CODE

;------------------------------------------------------------------------------
; ISR:  timer2_isr
;
;  Timer 2 is the symbol clock frequency reference which is synchronized to the RTCM
;  data. The regenerated bit clock is an even multiple (N_bitrate) of the interrupt rate. 
;  The clock is phase locked to the data bit transitions detected by the input capture 
;  ISR's. Reference phase adjustment is done once per data bit transition by presetting 
;  TMR2 to a correction value, which is derived by dividing the measured phase error 
;  (stored in 'Phase') by a constant. The divisor constant basically controls the loop 
;  bandwidth. 
;  The bit clock controls actions at two points: Data sampling intervals occur in-phase
;  with the bit transitions, while PLL phase adjustments occur out-of-phase (half-way)
;  with the transitions (making phase adjustment math & logic simpler).
;
;  NOTES:  The phase correction factor applied to TMR2 must never exceed the complement 
;          of the PR2 count. 
;
;          The regenerated symbol (bit) clock is output as a test point on I/O pin BIT_CLOCK_TP
;	   The demodulated data is output as a T.P. on I/O pin DATA_BIT_TP.
;
;  Assumptions:  Register Bank 0 is active

timer2_isr:

	; 'tmr2ext' is a down counter that resets to N_bitrate. A zero count corresponds to 
	; phase alignment with the data bit transition, while a count of N_HalfPhase is the 
	; reference point for the phase locking logic.

	decfsz	tmr2ext,F	; Test for in-phase with bit transitions, which is the data sample point.
	b	nobit		;   Jump if no

	;----------------------------------------------------------------------
	; In-phase with data bit transitions. Process most recent data bit and 
	; reset sampling for next bit.
	 
	bsf	BIT_CLOCK_TP		; Set symbol clock I/O pin high
	movfw	N_bitrate
	movwf	tmr2ext			; Reset counter
	
	; Determine the state of the last data bit, store it, and output it on I/O pin
	rlf	DataSum+1,W		; If (DataSum < 0 ) then CF = 1 else CF = 0
	skpc				;    CF is inverted bit state
	b	t000
	bcf	DataFlags,BIT_LEVEL
	bcf	DATA_BIT_TP
	b	t001
t000:	bsf	DataFlags,BIT_LEVEL
	bsf	DATA_BIT_TP

t001:	clrf	DataSum			; Reset accumulator to begin sampling next bit
	clrf	DataSum+1
		
	; Send data bit (normal mode only)
	; The bit accumulator is shifted right to make room for the new bit, which is stored in 
	; the b5 position. When 6 bits have been accumulated, the byte is sent to the UART.

	btfsc	SysState,TESTMODE
	b	noxmt			; Data isn't output in test mode

	clrc
	rrf	TxChar,F		; Make room for new bit
	btfsc	DataFlags,BIT_LEVEL
	bsf	TxChar,5		; If new bit is 1, set 

	; Check to see if we're ready to send a byte to the GPS 
	decfsz	BitCnt,F
	b	noxmt

	bsf	TxChar,6		; Set bit 6 (RTCM-104 std for NRZ byte), b7 is clear
	movfw	TxChar
	call	putch
	banksel	TxChar
	clrf	TxChar
	movlw	RTCM_BITS
	movwf	BitCnt

noxmt:
	btfss	PslibFlags,C1MSG    ;Before exiting isr check PslibFlags
	b	binmsg0 	    ;to see if a Starlink C1 freq/bps response
txc1msg:			    ;message should be sent
	bcf	PslibFlags,C1MSG    ;if so clear the flag saying it's pending
	movlw	0xc1		    ;and send it. Begin by saying C1
	call	putch		    ;
	banksel CurrentFreq	    ;Now isolate the ms 6 bits of CurrentFreq
	movfw	CurrentFreq	    ;
	movwf	tmp4		    ;
	movfw	CurrentFreq+1	    ;
	movwf	tmp5		    ;
	bcf	STATUS,C	    ;
	rlf	tmp4,F		    ;
	rlf	tmp5,F		    ;
	bcf	STATUS,C	    ;
	rlf	tmp4,F		    ;
	rlf	tmp5,W		    ;
	iorlw	0xc0		    ;OR that with 0xC0 and send it.
	call	putch		    ;
	banksel CurrentFreq	    ;Then send the ls 6 bits
	movfw	CurrentFreq	    ;
	iorlw	0xc0		    ;also OR'ed with 0xC0
	call	putch		    ;
	banksel CurrentBPS	    ;
	movfw	CurrentBPS	    ;
	call	putch		    ;
	movlw	0xf0		    ;then send F0
	call	putch		    ;
	movlw	0xf4		    ;and F4
	call	putch		    ;
binmsg0:			    ;Check PslibFlags to see if a starlink C5
	banksel PslibFlags	    ;binary status response is pending
	btfss	PslibFlags,C5MSG    ;
	b	nopll		    ;if not just exit ISR
txc5msg:			    ;
	bcf	PslibFlags,C5MSG    ;otherwise clear flag to say message sent
	movlw	0xc5		    ;and begin by sending C5
	call	putch		    ;
	banksel SysState	    ;
	btfsc	SysState,LOCKED     ;Check if receiver locked
	b	c5locked	    ;
c5nlockd:			    ;if not send signal level of 1 indicating
	movlw	0xc0		    ;not receiving
	call	putch		    ;
	movlw	0xc1		    ;
	call	putch		    ;
	b	txc50		    ;then go send SNR value
c5locked:			    ;If reciever locked send
	movlw	0xc3		    ;signal level of 200 DBuV
	call	putch		    ;(any value you wish)
	movlw	0xc8		    ;
	call	putch		    ;
txc50:				    ;send bogus value for SNR of 20Db
	movlw	0xd4		    ;(any value you wish)
	call	putch		    ;
	banksel PslibFlags	    ;
	b	nopll		    ;


	;----------------------------------------------------------------------
	; Test to see if we are at the 180 deg phase point of the bit clock, 
	; where the PLL logic is performed, and station lock is evaluated.

nobit:	clrc
	rrf	N_bitrate,W	; Load 1/2 the bit-rate multiplier into W and check for match
	subwf	tmr2ext,W
	bnz	nopll


	; Begin processing symbol period timeout. Reinitialize counters and toggle I/O pin

	bcf	BIT_CLOCK_TP		; Set symbol clock I/O pin low

	clrf	BitClock
	clrf	BitClock+1
		
	; Check to see if a bit transition was detected. If one was, calculate the phase 
	; error, which is equal to the difference between the time of the detected bit 
	; transition and half the bit clock period. The value is divided by 32 and then 
	; subtracted from the current timer2 count.

	btfss	DataFlags,TRANSITION	; Check if bit transition was detected
	b	t2done			;   If not, phase adjustment is skipped

	movfw	HalfTime		; Phase -= HalfTime
	subwf	Phase,F
	skpc				;   CF = 0 if borrow
	decf	Phase+1,F
	movfw	HalfTime+1
	subwf	Phase+1,F

	rlf	Phase,F			; Shift left 3 bits and then use the upper 
	rlf	Phase+1,F		;   8 bits (equiv to right shift 5 bits).
	rlf	Phase,F
	rlf	Phase+1,F
	rlf	Phase,F
	rlf	Phase+1,W		; W = Phase (error) / 32
	
	subwf	TMR2,F
	bcf	DataFlags,TRANSITION

	; Use timer correction value to determine if a station is locked. Each count  
	; represents phase errors of about a half millisecond. Accept absolute values
	; that are equal to or less than N_bitrate. 'MaxLockErr' has been initialized 
	; to ~(N_bitrate * 2).
	addwf	N_bitrate,W		; normalize acceptable range to 0 to (N_bitrate * 2)
	addwf	MaxLockErr,W		; Will set C if value is outside acceptable range
	bc	b001
	incf	LockDet,F		; Phase was good -- increase lock accumulator by small amount
	btfsc	LockDet,6		; Stop at 64 
	decf	LockDet,F
	b	b002

b001:	movlw	D'4'			; Phase was bad -- Decrease lock accumulator by larger amount
	subwf	LockDet,F		; Will clear C if wrap past 0
	skpc
	clrf	LockDet

	; Now check lock detect accumulator to determine whether a station is locked or not, and
	; set system status bit accordingly
b002:	movlw	D'15'			; lock detect threshold.
	subwf	LockDet,W		; Will set C if above threshold (lock is assumed).
	bnc	b003
	bsf	SysState,LOCKED		; Lock established
	b	b004
b003:	bcf	SysState,LOCKED		; Not locked
b004:	b	t2done


	;-- No PLL processing so just update the bit clock timer.
nopll:	movlw	TMR2_CNT
	addwf	BitClock,F
	addcf	BitClock+1,F

	
t2done:	bcf	PIR1,TMR2IF		; Clear interrupt flag
	goto	ISR_return



;------------------------------------------------------------------------------
; ISR:  Cap1_isr, Cap2_isr
;
;  The input capture ports are set up to capture the signal rising edge on one, 
;  and the falling edge on the other. Signal direction doesn't matter, so both 
;  ISR's perform identical functions. 
;
;  Assumptions:  Register Bank 0 is active

Capture_isr	MACRO	CntRegH, CntRegL, IntReg, IntFlg

	movfw	CntRegH		; Get MSB of CCPx reg
	movwf	tmp0
	movfw	CntRegL		; Get LSB of CCPx reg
	movwf	tmp1
	call	process_sig
	bcf	IntReg,IntFlg	; Clear interrupt flag
	goto	ISR_return

	ENDM

;-------------------------------------------

Cap1_isr:
	Capture_isr	CCPR1H,CCPR1L,PIR1,CCP1IF
	
Cap2_isr:
	Capture_isr	CCPR2H,CCPR2L,PIR2,CCP2IF





;------------------------------------------------------------------------------
; Local Function:  process_sig  (part of CCPx ISR's)
;
;  This function processes the data from the Input Capture interrupts, which
;  occur at each half period of the input signal (approx every 1000 us). Two
;  primary operations are performed.
;
;  The first operation performed is to accumulate the sum of the differences between
;  each raw signal period and the carrier baseline, which is used in making a "most likely"
;  bit state determination on the data stream by the timer2_isr function.
;
;  The other operation is to detect data stream state transitions in the modulated signal 
;  for use in phase locking the data bit clock (timer2_isr function). Transition detection
;  causes the current bit clock timer value to be stored so a phase error can be calculated 
;  and applied by the PLL logic.
;
;  The initial period calculation is done using 16 bits, then is clipped and normalized
;  to an an 8-bit period count which represents the number of 2 microsecond tics offset from 
;  800 microseconds. This gives a range of 800 us to 1310 us, which corresponds with signals 
;  between 382 and 625 Hz, which is slightly wider than the hardware pass-band. 
;
;  Since the MSK signal resembles FSK, data state changes can be sensed by detecting when
;  the signal period transitions above or below the baseline (carrier) signal period. To
;  reduce sensitivity to noise, the signal is filtered by averaging the 4 most recent period 
;  values. This is done by subtracting the oldest value from a running sum and adding the 
;  new value, then dividing by 4.
;
;  The carrier Baseline period is determined by averaging the last 1024 period values. 
;  The calculation is simplified by using another running sum (24 bit) and subtracting
;  the current Baseline then adding the new period, and dividing by 1024.
;
;  Data state transitions are identified by detecting when the averaged signal period crosses 
;  the carrier Baseline. This causes the bit clock timer to be sampled and stored in the
;  module variable 'phase'.
;
;  INPUT:	tmp0:tmp1	Hibyte:Lobyte of captured timer value
;
;  NOTE:  The filtered signal data state used for detecting bit transitions is output as a
;	  test point on I/O pin RAW_DATA_TP

process_sig:

	; The time period between the previous input capture and the current one is
	; calculated, then the value is clipped to fit into 8 bits
	;-- Get new count and subtract old count to get delta-T period
	movfw	LastICL		; Previous count low byte
	subwf	tmp1,W		; Subtract from new low byte
	movwf	tmp3		; Store delta-T low byte
	skpc			; NC if new-Lo < last-Lo
	incf	LastICH,F	;  "borrow"
	movfw	LastICH
	subwf	tmp0,W
	movwf	tmp2		; Store delta-T hi byte

	;-- Save new count as as previous, for next time
	movfw	tmp0
	movwf	LastICH
	movfw	tmp1
	movwf	LastICL
	; tmp0, tmp1 are now free for use

	;-- Check for underflow (delta-T < MIN_DT)
	movfw	tmp2		; Make "working" copy of delta-T hi
	movwf	tmp0

	movfw	tmp3
	addlw	low -MIN_DT
	skpc			; NC if dt_lo < low MIN_DT
	decf	tmp0,F		;  "borrow"
	movfw	tmp0
	addlw	-(high MIN_DT)
	skpnc			; NC if delta-T < MIN_DT
	goto	d000
	; Underflow. Clip to minimum level
	movlw	low MIN_DT
	movwf	tmp3
	goto	d010

d000:	;-- Check for overflow (delta-T > MAX_DT)
	movfw	tmp3
	sublw	low MAX_DT
	skpc			; NC if dt_lo > MAX_DT
	incf	tmp2,F
	movfw	tmp2
	sublw	high MAX_DT
	skpnc			; NC if delta-T > MAX_DT
	goto	d010
	; Overflow. Clip to maximum level
	movlw	low MAX_DT
	movwf	tmp3


d010:	; Convert the clipped delta-T low byte to an 8-bit unsigned offset from 
	; 800 microseconds. The value has a resolution of 2 us. 

	movfw	tmp3			; Subtract old count from new count
	addlw	low -(BASE_PER_CNT / US_PER_TIC)	; Normalize to 800us offset
	movwf	tmp0

	; Subtract oldest period from sum used in averaging then push "period stack" to 
	; make room for new value

	movfw	PerStk+3
	subwf	AveSum,F
	skpc			; CF = 0 if borrow
	decf	AveSum+1,F
	movfw	PerStk+2
	movwf	PerStk+3
	movfw	PerStk+1
	movwf	PerStk+2
	movfw	PerStk+0
	movwf	PerStk+1


	; Store new normalized period on stack then average it with previous readings to filter 
	; out noise. Use sum which already had oldest value subtracted.

	movfw	tmp0			; Retrieve normalized period
	movwf	PerStk+0		; Save it on stack
	addwf	AveSum,F		; Add to running sum
	addcf	AveSum+1,F
	rrf	AveSum+1,W		; Divide by 4 and save in AvePeriod
	movwf	tmp1
	rrf	AveSum,W
	movwf	AvePeriod
	rrf	tmp1,W			; put b0 in CF for next rotate
	rrf	AvePeriod,F

	; Calculate carrier Baseline period. Subtract current Baseline period from running sum
	; then add new value. New Baseline is the sum / 1024 (upper 16 bits of the sum right 
	; shifted twice).

	movfw	Baseline
	subwf	BaseSum,F
	bc	ps001			; Borrow from middle byte if required
	movlw	1
	subwf	BaseSum+1,F
ps001:	skpc				; CF = 0 if borrow
	decf	BaseSum+2,F

	movfw	PerStk+0		; Get current period value 
	addwf	BaseSum,F
	bnc	ps002
	movlw	1			; Add carry to middle byte
	addwf	BaseSum+1,F
ps002:	addcf	BaseSum+2,F
	rrf	BaseSum+2,W
	movwf	tmp1
	rrf	BaseSum+1,W
	movwf	Baseline
	rrf	tmp1,W
	rrf	Baseline,F

	; -- Compare averaged period with the Baseline to see if the data state has changed.
	clrf	tmp2
	movfw	Baseline
	subwf	AvePeriod,W		; CF is set to current data state
	skpnc
	bsf	tmp2,DATA_STATE

	movfw	DataFlags
	xorwf	tmp2,F			; DATA_STATE flag is now 1 if bit level changed
	btfss	tmp2,DATA_STATE
	b	tddone			;  Jump if no change

 	;-- Process data state change. Get bit clock count for PLL processing.
	movfw	BitClock		; Copy current counter into Phase variable,
	movwf	Phase			;   which will then have timer2 count added.
	movfw	BitClock+1
	movwf	Phase+1

	btfsc	PIR1,TMR2IF		; Check for timer2 wrap-around
	b	t2wrap
	movfw	TMR2			; No wrap, get current count
	btfss	PIR1,TMR2IF		; Verify timer didn't now just wrap 
	b	t2ok
t2wrap:	movfw	TMR2			; Timer wrapped. Get current count and add terminal count value
	addlw	TMR2_CNT
	addcf	Phase+1,F		; Add Carry 
t2ok:	addwf	Phase,F			; Add timer count
	addcf	Phase+1,F

	;-- Update DataFlags flags
	movlw	1<<DATA_STATE
	xorwf	DataFlags,F		; Toggle bit level bit
	bsf	DataFlags,TRANSITION	; Set flag for bit clock PLL to adjust phase


	;-- Output raw data state on I/O pin
	btfsc	DataFlags,DATA_STATE
	b	setpin
	bcf	RAW_DATA_TP
	b	tddone
setpin:	bsf	RAW_DATA_TP

	;----------------------------------------------------------------------
	; Calculate the difference between the raw signal period and the baseline,
	; then add it to the data sum. Selecting which sample from the history
	; stack is used allows compensation for the phase lag introduced by the 
	; filtering function on the data state transition detection.

tddone:	clrf	tmp3		; tmp3 is used to extend subtraction to 16 bits
	movfw	Baseline
	subwf	PerStk+2,W	; Use value from 2 samples ago when 4-value averaging is done
	bnc	ps003		;  Sign extend to upper byte
	decf	tmp3,F
	
ps003:	addwf	DataSum,F
	addcf	DataSum+1,F
	movfw	tmp3
	addwf	DataSum+1,F
	
	return




;------------------------------------------------------------------------------
; Function:  SetBitRate
;
;  Initializes module variables for the selected data bit rate. Any received bits 
;  not yet sent to the GPS are discarded.
;
;  INPUT    W:	Bit rate identifier (BPS_200, BPS_100, BPS_50)
;
;	Modified to set global var CurrentBPS to
;	      0xD5 = 50
;	      0xD6 = 100
;	      0xD7 = 200
;	for use in responding to $PSLIB,,,K sentence.
;


SetBitRate:
	banksel N_bitrate
	movwf	N_bitrate
	movwf	tmr2ext
	movwf	MaxLockErr
	movwf	tmp0

	banksel CurrentBPS		;Set value of CurrentBPS for status msg
	movlw	0xD5			;Assume 0xD5. Starlink's code for 50bps
	movwf	CurrentBPS		;
	banksel tmp0			;Get BPS value passed in
	movfw	tmp0			;
	sublw	BPS_50			;and compare to 50bps constant
	bz	sbr0			;if 50bps we are done. continue
	banksel CurrentBPS		;
	incf	CurrentBPS,F		;if not assume 0xD6. (100bps)
	banksel tmp0			;
	movfw	tmp0			;
	sublw	BPS_100 		;
	bz	sbr0			;if 100 bps we are done. continue
	banksel CurrentBPS		;
	incf	CurrentBPS,F		;Otherwise it must be 0xD7.(200bps)
sbr0:
	banksel MaxLockErr
	clrc
	rlf	MaxLockErr,F
	comf	MaxLockErr,F		; Set to ~(N_bitrate * 2)

	;-- Init HalfTime to 1/2 the number of timer2 tics per bit period
	clrc
	rrf	tmp0,F
	clrf	HalfTime
	clrf	HalfTime+1
	movlw	TMR2_CNT
sbr00:	addwf	HalfTime,F
	addcf	HalfTime+1,F
	decfsz	tmp0,F
	b	sbr00

	clrf	LockDet
	clrf	BitClock
	clrf	BitClock+1
	clrf	DataFlags
	clrf	TxChar
	movlw	RTCM_BITS
	movwf	BitCnt
	return


;------------------------------------------------------------------------------
; Function:  InitDemod
;
;  Startup initialization to begin collecting raw signal half-period data. 
;  The HW timers and interrupts for the Input Capture ports are initialized here.

InitDemod:
	;-- Initialize module variables
	banksel	BaseSum
	movlw	INIT_BASE_SUM & 0FF	; 24-bit BaseSum 
	movwf	BaseSum
	movlw	(INIT_BASE_SUM / 100) & 0FF
	movwf	BaseSum+1
	movlw	(INIT_BASE_SUM / 10000) & 0FF
	movwf	BaseSum+2

	movlw	INIT_BASELINE
	movwf	Baseline
	movwf	PerStk			; period stack -- Set to nominal carrier period
	movwf	PerStk+1
	movwf	PerStk+2
	movwf	PerStk+3


	movlw	(INIT_BASELINE * 4) & 0FF
	movwf	AveSum
	movlw	((INIT_BASELINE * 4) / 100) & 0FF
	movwf	AveSum+1

	;-- Set up Timer1
	banksel	T1CON
	movlw	INI_T1CON
	movwf	T1CON

	;-- Set up CCP1 to capture one edge
	banksel	CCP1CON
	movlw	INI_CCP1CON
	movwf	CCP1CON
	bcf	PIR1,CCP1IF	; Clear interrupt flag then enable interrupt
	banksel	PIE1
	bsf	PIE1,CCP1IE

	;-- Set up CCP2 to capture the other edge
	banksel	CCP2CON
	movlw	INI_CCP2CON
	movwf	CCP2CON
	bcf	PIR2,CCP2IF	; Clear interrupt flag then enable interrupt
	banksel	PIE2
	bsf	PIE2,CCP2IE

	;-- Set up Timer2 to generate interrupts as symbol clock reference frequency
	banksel	T2CON
	movlw	INI_T2CON	; Set up timer and pre/post-scalers
	movwf	T2CON
	bcf	PIR1,TMR2IF	; Make sure interrupt flag is clear
	banksel	PR2
	movlw	TMR2_CNT - 1
	movwf	PR2		; Enable the interrupt
	bsf	PIE1,TMR2IE


	return


	END
