; DGPS Radio Controller Software -- PIC16C73B, 16C76
; Copyright (c) 1998, 1999 Rich Heineck, All rights reserved.
; This program can be freely used for non-commercial purposes, and carries no warranty of any kind.
;
; File:		NormMode.asm
; Date:		05/12/98
; Language:	Microchip 16CXX Assembler
;
;   This module implements the normal operating mode background tasks, which includes monitoring
;   for and execution of tuning commands from the GPS.
;
;  Change History
;	6/9/98	  Changed mTuningCmd function to use AtoFP1Num for reading tuning frequency, which uses 
;		   fixed point format, allowing tuning of international stations on 0.5 kHz boundaries.
;		  Added LOCK LED control code.
;
;	10/01/98  Added serial comm rate select via RB1 to NormalMode startup. Default is 4800 bps.
;	6/13/99   Added support for 16C73B - eliminated shared data space. 
;   1/7/00    $PSLIB J & K command support added (Landrum Haddix).
;------------------------------------------------------------------------------
	INCLUDE		<pic_hdr.inc>
	INCLUDE		<dgps.inc>
	LIST		n=114, x=ON


	GLOBAL	NormalMode, tmr0_isr
	EXTERN	SetBitRate, SetFreq, StoreStation, ISR_return, PslibFlags


; Time periods for controlling LOCK LED
TMR0_INT_RATE	EQU	D'65'		; Approx number of ms between interrupts
NOLOCK_PERIOD	EQU	D'2000' / TMR0_INT_RATE	; 2 seconds
LOCK_PERIOD	EQU	D'666' / TMR0_INT_RATE	    ; 2/3 second
NOLOCK_ON_PER	EQU	2
LOCK_ON_PER	EQU	1




	UDATA
; Module variables
CLptr		RES	1	; Current pointer into command line

; Local Variables for NormalMode
ch		RES	1

; Local Variables for mTuningCmd
NewRate		RES	1
NewFreq		RES	2

; Local Variables for mNMEA_check
tmptr		RES	1
cksum		RES	1
tmpch		RES	1

BANK0	UDATA
BlinkTimer	RES	1	; Used in timing LOCK LED blinking

	CODE


;------------------------------------------------------------------------------
; Local Function:  mNMEA_check
;
;   Verifies the checksum for the NMEA command, if a checksum is included. NMEA
;   command sentences containing a checksum end with "*hh" (where hh is a two 
;   char hex number). The checksum is the exclusive-OR value of all characters
;   after the '$' and before the '*'.
;
;   INPUT:	W	: Offset in command line to first character past '$'
;   OUTPUT:	Z Flag	: Set if checksum is OK (or isn't in sentence), or cleared
;			  if checksum exists but doesn't match.

mNMEA_check:
	movwf	tmptr		; working copy of offset into command
	clrf	cksum

	; Add all characters in command sentence to 'cksum' accumulator until '*' is found
lp1:	movfw	tmptr
	call	GetCmdChr
	banksel	tmptr
	incf	tmptr,F
	movwf	tmpch
	sublw	'*'
	bz	dochk
	movfw	tmpch
	skpnz			; If no '*' is found (null terminator detected), checksum is not used,
	return			;   so return with Z set

	xorwf	cksum,F
	b	lp1

	; Get checksum from command and put it into 'tmpch'
dochk:	movfw	tmptr
	call	GetCmdChr	; Get first digit of checksum
	banksel	tmptr
	incf	tmptr,F
	addlw	-('0')		; Result is 00 to 09, or 11 to 16 (hex)
	movwf	tmpch
	sublw	9		; C = 1 if char was '0' - '9'
	bc	b000		;  
	movlw	('A' - '9' - 1)
	subwf	tmpch,F

b000:	swapf	tmpch,F		; Move to upper nibble
	movfw	tmptr
	call	GetCmdChr	; Get last digit of checksum
	banksel	tmptr
	addlw	-('0')		; Result is 00 to 09, or 11 to 16 (hex)
	addwf	tmpch,F
	sublw	9		; C = 1 if char was '0' - '9'
	bc	b001
	movlw	('A' - '9' - 1)
	subwf	tmpch,F

	; Compare accumulated checksum to command's checksum and return result in Z
b001:	movfw	tmpch
	subwf	cksum,W
	return




;------------------------------------------------------------------------------
; Local Function:  mTuningCmd
;
;   Parses the tuning parameters from the propriatary NMEA command and changes
;   the frequency & bit rate. Interrupts are disabled during function execution.
;
;   INPUT:	CLptr	: Offset in command line to first digit in frequency parameter

mTuningCmd:

	DisableInt
	;-- Get frequency parameter and convert into 16-bit number, then set frequency
	movfw	CLptr
	call	AtoFP1Num	; Number is returned in arg2:arg1
	banksel	CLptr
	movwf	CLptr
	banksel	arg1
	movfw	arg1		; Temporarily save new frequency
	banksel	NewFreq
	movwf	NewFreq
	banksel	arg2
	movfw	arg2
	banksel	NewFreq
	movwf	NewFreq+1
	call	SetFreq		; Freq is passed in arg2:arg1
	banksel	CLptr

	;-- Get first digit of bit rate parameter to ID rate, then set rate.
	movlw	','
	banksel	arg1
	movwf	arg1
	banksel	CLptr
	movfw	CLptr
	call	FindChr		; Advance to ',' delimiter first
	addlw	1		; Bypass ',' to first digit
	call	GetCmdChr
	banksel	CLptr
	movwf	ch

	sublw	'2'
	bnz	not_200
  	movlw	BPS_200
	b	set_rate	
  	
not_200:
	movfw	ch
	sublw	'1'
	bnz	not_100
  	movlw	BPS_100
	b	set_rate	
	
not_100:
  	movlw	BPS_50
	
set_rate:
	movwf	NewRate		; Temporarily save
	call	SetBitRate	; Rate code is passed in W
	banksel	NewFreq

	; Store new tuning parameters in EEPROM
	movfw	NewFreq
	banksel	arg1
	movwf	arg1
	banksel	NewFreq
	movfw	NewFreq+1
	banksel	arg2
	movwf	arg2
	banksel	NewRate
	movfw	NewRate
	call	StoreStation	; Freq is passed in arg2:arg1, bps code in W
	banksel	CLptr
	EnableInt
	return







;------------------------------------------------------------------------------
; Function:  NormalMode
;
;   Normal operating mode. Finish initialization then drop into background processing
;   loop, which scans the serial port for incoming commands.

NormalMode:

	;-- Initialize serial I/O. Default to 4800 bps. If RB1 is grounded use 9600.
	banksel	PORTB
	movlw	INI_SPBRG48
	btfss	COM_RATE_SEL
	movlw	INI_SPBRG96
	call	init_uart

	;-- Enable Timer 0 interrupt
	bcf	INTCON,T0IF
	bsf	INTCON,T0IE

	;-- Enable interrupts
	bsf	INTCON,PEIE
	bsf	INTCON,GIE

	call	InitCmd		; Set up to begin receiving serial commands
		


nm_loop:	;  Normal Mode background (bg) loop


	;--------------------------------------------------
	;  Accumulate an NMEA command line from serial port. When a line has been received, 
	;  read the command nmenonic to see if it might be a tuning command. Maintain CLptr, 
	;  which is the offset into the command line of the next unparsed character so the 
	;  command function can extract the command line parameters.

	call	CollectCmd
	bnz	nm_nxt

	;-- Get first non-space character in command line
	banksel	CLptr
	clrf	CLptr
lp0:	movfw	CLptr
	call	GetCmdChr
	banksel	CLptr
	movwf	ch
	incf	CLptr,F
	sublw	' '
	bz	lp0

	movfw	ch		; first command character

	;-- Test for NMEA propriatary command "$P....."
	sublw	'$'
	bnz	not_nmea	; Jmp if not NMEA sentence to check for non-NMEA commands
	movfw	CLptr
	call	GetCmdChr
	sublw	'P'
	bnz	cmd_done	; Not propriatary ... discard line

	banksel	CLptr
	movfw	CLptr
	call	mNMEA_check	; Verify command's checksum 
	bnz	cmd_done	;  Z = 0 if checksum exists and doesn't match.

	;-- Test for PSLIB tuning command. Example:  "$PSLIB,323.0,200*5A"
	;   Non-tuning PSLIB commands start with "$PSLIB,,,"
	incf	CLptr,F
	movfw	CLptr
	call	GetCmdChr
	sublw	'S'
	bnz	cmd_done	; Jmp if not supported command

	banksel	CLptr
	incf	CLptr,F
	movfw	CLptr
	call	GetCmdChr
	sublw	'L'
	bnz	cmd_done	; Jmp if not supported command

	banksel	CLptr
	incf	CLptr,F
	movfw	CLptr
	call	GetCmdChr
	sublw	'I'
	bnz	cmd_done	; Jmp if not supported command

	banksel	CLptr
	incf	CLptr,F
	movfw	CLptr
	call	GetCmdChr
	sublw	'B'
	bnz	cmd_done	; Jmp if not supported command

	banksel	CLptr
	incf	CLptr,F		; Bypass first ','
	incf	CLptr,F
	movfw	CLptr
	call	GetCmdChr
	sublw	','
	bz	psljk		;Jmp if ',' -- Not a tuning command
				;could be $PSLIB,,,J or K
	call	mTuningCmd
	goto	cmd_done

psljk:
	banksel CLptr		;Look for another ','
	incf	CLptr,F 	;
	movfw	CLptr		;
	call	GetCmdChr	;
	sublw	','		;
	bnz	cmd_done	;if no third comma can't be pslib,,,j or k
	banksel CLptr		;Look for 'J' or 'K'
	incf	CLptr,F 	;
	movfw	CLptr		;
	call	GetCmdChr	;
	sublw	'K'		;
	bz	pslibk		;if message is $PSLIB,,,K go set flag for
	banksel CLptr		;response message request
	movfw	CLptr		;
	call	GetCmdChr	;Re-get the character and check for a 'J'
	sublw	'J'		;
	bz	pslibj		;if message is $PSLIB,,,J go set flag
	b	cmd_done	;else command was nothing we respond too
pslibk:
	banksel PslibFlags	;
	bsf	PslibFlags,C1MSG;
	b	cmd_done	;
pslibj:
	banksel PslibFlags	;
	bsf	PslibFlags,C5MSG;
	b	cmd_done	;

not_nmea:
	banksel	ch
	movfw	ch		;--- Put non-NMEA commands to be supported here
	goto	cmd_done

cmd_done:
	call	InitCmd		; Begin receiving new command


nm_nxt:	
	;--------------------------------------------------
	; Monitor status of LOCKED flag to determine if a station is locked, and
	; control the LOCK LED blink pattern to indicate the status.

	banksel	_bank0_
	btfss	SysState,LOCKED
	b	nolock

	; LOCK established: Blink at LOCK_PERIOD interval
	movlw	LOCK_PERIOD
	subwf	BlinkTimer,W		; C set if period overflow
	bc	n000

	movlw	LOCK_ON_PER
	subwf	BlinkTimer,W		; C set if time to turn LED off
	skpnc			
	bsf	LOCK_LED		;  set LED off
	b	bl_end
n000:	clrf	BlinkTimer		; Timer overflowed, reset it and turn the LED on
	bcf	LOCK_LED
	b	bl_end


	; No station lock: Blink at NOLOCK_PERIOD interval
nolock:	movlw	NOLOCK_PERIOD
	subwf	BlinkTimer,W		; C set if period overflow
	bc	n001

	movlw	NOLOCK_ON_PER
	subwf	BlinkTimer,W		; C set if time to turn LED off
	skpnc			
	bsf	LOCK_LED
	b	bl_end
n001:	clrf	BlinkTimer		; Timer overflowed, reset it and turn the LED on
	bcf	LOCK_LED

bl_end:


	goto	nm_loop



;------------------------------------------------------------------------------
; ISR:  tmr0_isr
;
;   Timer 0 is set up to interrupt every 65.536 ms. It's purpose is to run the 
;   LOCK LED blink timer. BANK0 is selected upon entry.

tmr0_isr:
	incf	BlinkTimer,F
	bcf	INTCON,T0IF
	goto	ISR_return




	END
