; DGPS Radio Controller Software -- PIC16C73B, 16C76
; Copyright (c) 1998, 1999 Rich Heineck, All rights reserved.
; This program can be freely used for non-commercial purposes, and carries no warranty of any kind.
;
; File:		PLLsynth.asm
; Date:		02/21/98
; Language:	Microchip 16CXX Assembler
;
;   This module contains the functions that program the PLL frequency synthesizer used for the 
;   local oscillator. 
;
;  Change History
;	6/9/98	  Changed SetFreq function to take input argument in 500 Hz units to allow
;		   tuning international stations.
;	6/13/99   Added support for 16C73B - eliminated shared data space. 
;------------------------------------------------------------------------------
	INCLUDE		<pic_hdr.inc>
	INCLUDE		<dgps.inc>
	LIST		n=114, x=ON

	EXTERN	CurrentFreq
	GLOBAL	InitSynth, SetFreq


	CONSTANT	REF_FREQ = D'500'	;!!! If this is changed, SetFreq will need to be updated
	CONSTANT	REF_DIVIDER = (((SYS_CLOCK / REF_FREQ) << 1) | 1)	; Ready to load
	CONSTANT	INI_FREQ = (((D'285' * 2) + 1) << 1)

INI_SSPCON	EQU	1<<SSPEN | 1<<CKP | B'0000'	; SPI, clock = Fosc/4, clock idle high
INI_SSPSTAT	EQU	0				; Data xfer at clock RE




	CODE


;------------------------------------------------------------------------------
; Function:  LoadSynth
;
;	Shifts a count value into the MC145157 and then latches it into the
;	count register encoded in the passed data word.
;
;	ASSUMPTIONS:
;	    The true 14-bit data is LSB justified (bits0..13 in arg2:arg1)
;	    The data contains the MC145157 control bit for register selection.
;
;	INPUT:	arg1	: Low byte of 16-bit data
;		arg2	: Hi byte

LoadSynth:
	
	;-- Transfer hi byte first
	banksel	SSPBUF
	movfw	SSPBUF		; Dummy read to clear BF bit
	banksel arg2
	movfw	arg2
	banksel	SSPBUF
	movwf	SSPBUF		; Starts transfer
	banksel	SSPSTAT
lp0:	btfss	SSPSTAT,BF	; Wait here until completed
	b	lp0

	;-- Transfer low byte
	banksel	SSPBUF
	movfw	SSPBUF		; Dummy read to clear BF bit
	banksel	arg1
	movfw	arg1
	banksel	SSPBUF
	movwf	SSPBUF		; Starts transfer
	banksel	SSPSTAT
lp1:	btfss	SSPSTAT,BF	; Wait until completed
	b	lp1

	;-- Pulse MC145157 ENB to latch data
	banksel	PORTB
	bsf	SYN_ENB
	bcf	SYN_ENB
	return



;------------------------------------------------------------------------------
; Function:  SetFreq
;
;	Programs the PLL synthesizer (MC145157) local oscillator to receive the 
;	station at the specified frequency. The input units are kHz, and are in 
;	fixed point format, with one bit after the decimal, giving a resolution
;	of 500 Hz.
;
;	ASSUMPTIONS:
;	    The synthesizer reference frequency is 500 Hz.
;	    The desired mixer output is 500 Hz, produced by Freq[vco] - Freq[rf_in].
;
;	INPUT:	arg1	: Low byte of 16-bit station frequency (x500 Hz)
;		arg2	: Hi byte
;
;	Modified to set global var CurrentFreq to station frequency * 100HZ
;	for use in responding to $PSLIB,,,K sentence.
;
;

SetFreq:
	banksel arg1		;Update value of CurrentFreq for
	movfw	arg1		;arg1,arg2 contain freq / 500hz
	banksel CurrentFreq	;CurrentFreq is in units of freq /100hz
	movwf	CurrentFreq	;so we must set it to arg1,arg2 * 5
	banksel arg2		;So first get a copy of arg1,arg2
	movfw	arg2		;into CurrentFreq
	banksel CurrentFreq	;
	movwf	CurrentFreq+1	;
	bcf	STATUS,C	;
	rlf	CurrentFreq,F	;double it
	rlf	CurrentFreq+1,F ;
	bcf	STATUS,C	;
	rlf	CurrentFreq,F	;twice
	rlf	CurrentFreq+1,F ;
	bcf	STATUS,C	;
	banksel arg1		;
	movfw	arg1		;
	banksel CurrentFreq	;then add arg1,arg2
	addwf	CurrentFreq,F	;
	banksel arg2		;
	movfw	arg2		;
	banksel CurrentFreq	;Now CurrentFreq is arg1,arg2 * 5
	addwf	CurrentFreq+1,F ;

	;-- Convert station frequency into MC145157 VCO divider value. Since the 
	;   input value units match the reference frequency, the ratio is 1:1. 
	banksel	arg1
	movlw	1
	addwf	arg1,F		; Add 1 to produce 500 Hz mixer output.
	skpnc
	incf	arg2,F

	;-- Form MC145157 data word: b0 = control bit, b1..13 = divisor
	clrc			; Control bit: 0 selects the N-counter latch
	rlf	arg1,F
	rlf	arg2,F

	call	LoadSynth
	return



;------------------------------------------------------------------------------
; Function:  InitSynth
;
;	Initializes the Sync. Serial Port (SSP) for SPI, and programs the PLL 
;	synthesizer (MC145157) reference divider. It also programs the synth
;	N divider for the default frequency.

InitSynth:
	;-- Initialize the SSP
	movlw	INI_SSPCON
	banksel	SSPCON
	movwf	SSPCON

	movlw	INI_SSPSTAT
	banksel	SSPSTAT
	movwf	SSPSTAT

	banksel	arg1

	;-- Program MC145157 reference divider
	movlw	low REF_DIVIDER
	movwf	arg1
	movlw	high REF_DIVIDER
	movwf	arg2
	call	LoadSynth

	;-- Program MC145157 default N divider
	movlw	low INI_FREQ
	movwf	arg1
	movlw	high INI_FREQ
	movwf	arg2
	call	LoadSynth

	return

	END
